function data = data_generator(gr, pm)
% generate sparse synthetic datasets with group structure by b = A*x + e

% Input: groupSturcture gr, parameters set pm
%       pm.numSample: number of generated datasets: default value: 1
%       pm.numNonzeroGroup: number of nonzero groups: default value: 7
%       pm.obs: number of observations: default value: 100
%       pm.noiseLev: noise level: default value: 0.1

% Output: data
%       data.A: feature matrices A, size:obs*dim*numSample
%       data.b: response vectors b, size:obs*numSample
%       data.xStar: ground truths, size:dim*1

% Author: Chengyu Ke
% Date: 3/15/2021

%% Parameters

numSample = 1;
if isfield(pm,'numSample'); numSample = pm.numSample; end
numGroup = length(gr);
numNonzeroGroup = 7;
if isfield(pm,'numNonzeroGroup'); numNonzeroGroup = pm.numNonzeroGroup; end
if numNonzeroGroup > numGroup
    error('num of nonzero groups should be less than total num of groups')
end
obs = 100;
if isfield(pm,'obs'); obs = pm.obs; end
noiseLev = 0.1;
if isfield(pm,'noiseLev'); noiseLev = pm.noiseLev; end

%% Data generations

dim = sum(gr);
xStar = zeros(dim,numSample);

indexInGroup = ones(numGroup,2); 
indexInGroup(1,2) = indexInGroup(1,1)+ gr(1) - 1;
for i = 2:numGroup
    indexInGroup(i,1) = indexInGroup(i-1,2) + 1;
    indexInGroup(i,2) = indexInGroup(i,1) + gr(i) - 1;
end

sampleSD = zeros(numSample,1);
A = zeros(obs,dim,numSample);
for i = 1:numSample
    % generate feature matrix A based on N(0,1)
    A(:,:,i) = normrnd(0,1,[obs,dim]);
    % genetate ground truth
    nonzeroGroupIndex = randsample(numGroup,numNonzeroGroup);
    for iNonzeroGroup = 1:numNonzeroGroup
        xStar(indexInGroup(nonzeroGroupIndex(iNonzeroGroup),1):indexInGroup(nonzeroGroupIndex(iNonzeroGroup),2),i) = unifrnd(-5,5,[gr(nonzeroGroupIndex(iNonzeroGroup)),1]);
    end
    sampleSD(i) = std(A(:,:,i)*xStar(:,i));    
end


sigmaNoise = noiseLev*mean(sampleSD);
b = zeros(obs,numSample);
for i = 1:numSample
    % generate b based on b = A*xStar + e
    b(:,i) = A(:,:,i)*xStar(:,i) + normrnd(0,sigmaNoise,[obs,1]);
end

data.A = A;
data.b = b;
data.xStar = xStar;
end
