function [x,output] = glasso_ADMM(A,b,w,gr,pm)
%min_x 1/(2*n) ||Ax-b||^2 + lambda * sum_k( sqrt(gr./w) ||x_k||_2)

% this function solves a group LASSO subproblem in group LOG problem
% Input: dictionary A, data b, group size gr, fixed group weight w>0, parameters set pm
%       pm.lambda: regularization paramter
%       pm.rho: penalty parameter for ADMM, default value: 0.01
%       pm.maxit: max inner iterations: default value: 5000
%       pm.tol: outer tolerace, default value: 1e-3
%       pm.abstol: abs tolerance for ADMM: default value: 1e-7
%       pm.reltol: rel tolerance for ADMM: default value: 1e-5
%       pm.intercept: whether the intercept is included, default value: true  
% Output: computed coefficients x
%       output.relerr: relative error of x_n and x_{n-1}
%       output.obj: objective function of x_n:
%       obj(x) = lambda |x|_1+0.5|Ax-b|^2
%       output.res: residual of x_n: norm(Ax-b)/norm(b)
%       output.err: error to the ground-truth: norm(x-xg)/norm(xg)
%       output.relerr: error between two neighboring iterates
%       output.time: computational time

% Originally provided by Yifei Lou
% Modified by Chengyu Ke, Miju Ahn
% Date: 3/15/2021

%% DCA for outer interation
[M,N] = size(A); start_time = tic; 
ng = length(gr);


%% parameters
lambda = 1e-5;  
maxit = 5*N;  
x0 = zeros(N,1); 
reltol = 1e-7; abstol = 1e-7; %1e-8, 1e-8
eps = 1e-16;
rho = 0.01;
hasintercept = true;

if isfield(pm,'lambda'); lambda = pm.lambda; end
if isfield(pm,'rho'); rho = pm.rho; end
if isfield(pm,'maxit'); maxit = pm.maxit; end
if isfield(pm,'x0'); x0 = pm.x0; end
if isfield(pm,'abstol'); abstol = pm.abstol; end
if isfield(pm,'reltol'); reltol = pm.reltol; end
if isfield(pm,'intercept'); hasintercept = pm.intercept; end

%% intercept
if hasintercept
     icp = 0;
end
%% pre-computing/initialize

AAt = A*A'; % option 2
L = chol( speye(M) + 2*AAt/M/rho, 'lower' ); 
L = sparse(L);
U = sparse(L');

%temp = rho*eye(N) + A'*A/M; % option 1

Atb = A'*b; 

x = x0;
z = zeros(N,1); v = z;

lambdak = lambda*sqrt(gr./w)/rho;

for it = 1:maxit
  
    %update x
    for j = 1:ng
        if j == 1
            first = 1;
        else
            first = first + gr(j-1);
        end
        last = first + gr(j) - 1;
        
        % x update
        x(first:last) = shrink(z(first:last) - v(first:last), lambdak(j));
    end
    
    % update z
    if hasintercept
        rhs = 2*(Atb - A'*(ones(M,1).*icp))/M + rho*(x+v);
    else
        rhs = 2*Atb/M + rho*(x+v);
    end   
    z = rhs/rho - 2*(A'*(U\(L\A*rhs)))/M/(rho^2); % option 2
    %z = temp\rhs; % option 1
    
    %update v
    v = v + x - z;
    
    %update icp
    if hasintercept
        icp = mean(b - A*z);
    end
    
    %horzcat(x,z,v)
    
    % stop conditions & outputs
    relerr = norm(x-z)/max([norm(x),norm(z),eps]);
    residual = norm(A*x - b);

    
    output.relerr(it) = relerr;
    %output.obj(it) = obj(A,b,x,gr,lambda);
    output.time(it) = toc(start_time);
    output.res(it) = residual;
    output.L0(it) = nnz(x);
    
    if  residual < abstol || relerr < reltol
        break;
    end
end
output.it = it;
output.totaltime = toc(start_time);
if hasintercept
   output.icp = icp;  
end




end

%% Shrinkage
function z = shrink(x, r)
    if x == 0
        z = 0;
    else
    z = x*max(1-r/norm(x),0);
    end
end

%% objective
function f = obj(A,b,x,gr,lambda)

    [M,~] = size(A);
    objval = 1/M*norm(A*x-b)^2;

    for j = 1:length(gr)
        if j == 1
            first = 1;
        else
            first = first + gr(j-1);
        end
        last = first + gr(j) - 1;

        % x update
        objval = objval + lambda*sqrt(gr(j))*norm(x(first:last));
    end
    f = objval;
end
