function [x, output] = glog_ADMM(A,b,gr,pm)
% min_x 1/(2* n) ||Ax-b||^2 + lambda * sum_k( sqrt(gr) glog(||x_k||_2, epsilon)))

% Input: dictionary A, data b, group size gr, parameters set pm
%       pm.varep: hyperparameter for group log-composite penalty: default value: 1e-3
%       pm.maxoutit: max outer iterations: default value: 15
%       pm.lambda: regularization paramter: default value: 1e-5
%       pm.rho: penalty parameter for ADMM, default value: 0.01
%       pm.x0: initial point, default value: vector 1
% Output: computed coefficients x
%       output.it: number of inner iterations for each outer iteration, size: maxoutit*1 
%       output.totalit: number of total inner iterations, i.e, sum(output.it)
%       output.totaltime: computational time
%       output.icp: intercept

% Originally provided by Yifei Lou
% Modified by Chengyu Ke, Miju Ahn
% Date: 3/15/2021

%% parameters

outreltol = 1e-6;
maxoit = 15;
varep = 1e-3;
lambda = 1e-5;
x0 = ones(size(A,2),1);
start_time = tic;

if isfield(pm,'maxoutit'); maxoit = pm.maxoutit; end
if isfield(pm,'varep'); varep = pm.varep; end
if isfield(pm,'x0'); x0 = pm.x0; end

% parameters for subproblem (group lasso)
pm2.reltol = 1e-7; pm2.abstol = 1e-7;
pm2.lambda = 1e-5;
pm2.maxit = 50;

if isfield(pm,'lambda'); pm2.lambda = pm.lambda; end
if isfield(pm,'lambda'); lambda = pm.lambda; end
if isfield(pm,'rho'); pm2.rho = pm.rho; end

[~,N] = size(A);
ng = length(gr);

x = x0;
wt = zeros(ng,1);

output.totalit = 0;
for it = 1:maxoit
    
    for j = 1:ng
        if j == 1
            first = 1;
        else
            first = first + gr(j-1);
        end
        last = first + gr(j) - 1;
        
        % update weight
        wt(j) = norm(x(first:last))^2 + varep;
    end
    
    % solve group lasso with current weight
    x_old = x;
    [x,output2] = glasso_ADMM(A,b,wt,gr,pm2);
    output.totalit = output.totalit + output2.it;
    
    relerr = norm(x-x_old,2);
    if  relerr < outreltol
        break;
    end
    
end

output.it = it;
output.totaltime = toc(start_time);
output.icp = output2.icp;
end


%% Group Lasso Objective
function f = obj(A,b,x,gr,wt,lambda)

    [M,~] = size(A);
    objval = 1/M*norm(A*x-b)^2;

    for j = 1:length(gr)
        if j == 1
            first = 1;
        else
            first = first + gr(j-1);
        end
        last = first + gr(j) - 1;

        % x update
        objval = objval + lambda*sqrt(gr(j)./wt(j))*norm(x(first:last));
    end
    f = objval;
end


